import { NextRequest, NextResponse } from 'next/server';

export async function GET(request: NextRequest) {
  const { searchParams } = new URL(request.url);
  const wikidataId = searchParams.get('wikidataId');

  if (!wikidataId || typeof wikidataId !== 'string') {
    return NextResponse.json({ error: 'wikidataId parameter is required' }, { status: 400 });
  }

  try {
    const sparqlQuery = `
      SELECT ?prop ?propLabel ?value ?valueLabel WHERE {
        wd:${wikidataId} ?prop ?statement.
        ?statement ?ps ?value.
        ?property wikibase:claim ?prop.
        ?property wikibase:statementProperty ?ps.
        SERVICE wikibase:label { 
          bd:serviceParam wikibase:language "en". 
          ?prop rdfs:label ?propLabel.
          ?value rdfs:label ?valueLabel.
        }
      }
    `;

    const url = `https://query.wikidata.org/sparql?query=${encodeURIComponent(sparqlQuery)}&format=json`;
    
    const response = await fetch(url, {
      headers: { 
        'Accept': 'application/json',
        'User-Agent': 'WikidataApp/1.0 (https://example.com/contact)'
      }
    });
    
    if (!response.ok) {
      throw new Error(`Wikidata SPARQL error: ${response.status}`);
    }
    
    const data = await response.json();
    const result: Record<string, string> = {};
    
    if (data.results?.bindings) {
      for (const binding of data.results.bindings) {
        const propUri = binding.prop?.value;
        const value = binding.valueLabel?.value || binding.value?.value || '';
        
        if (propUri) {
          const propId = propUri.split('/').pop() || '';
          result[propId] = value;
        }
      }
    }
    
    return NextResponse.json(result);
  } catch (error) {
    console.error('Error fetching Wikidata properties:', error);
    return NextResponse.json({});
  }
}